﻿// Copyright (c) Microsoft Corporation.  All rights reserved.

//------------------------------------------------------------------------------
// PLEASE NOTE:  Microsoft Corporation (or based on where you live, one of its affiliates) 
// licenses this supplement to you.  You may use it with each validly licensed copy of Microsoft 
// Visual Studio Pre-Release Code-Name "ORCAS" software (the “software”).  You may not use the 
// supplement if you do not have a license for the software.  The license terms for the software 
// apply to your use of this supplement. 
//
// Please note: As this software is distributed in Quebec, Canada, this notice is provided below in French.
// Remarque: Ce logiciel étant distribué au Québec, Canada, les termes de cette licence sont fournis ci-dessous en français.
// REMARQUE:  Microsoft Corporation (ou en fonction du lieu où vous vivez, l’un de ses affiliés) vous 
// accorde une licence pour ce supplément.  Vous pouvez utiliser une copie de ce supplément avec chaque 
// copie concédée sous licence du logiciel Microsoft Visual Studio Pre-Release Code-Name "ORCAS" (le « logiciel »).  
// Vous n’êtes pas autorisé à utiliser ce supplément si vous n’êtes pas titulaire d’une licence pour le logiciel.  
// Les conditions de licence pour le logiciel s’appliquent à l’utilisation de ce supplément.  
//------------------------------------------------------------------------------
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Configuration;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Data;
using System.Data.Objects;

using NorthwindLib;

namespace RichClient
{
    public partial class WinAppSample : Form
    {
        // declare a form-wide (or app wide) object context. Note
        // that there is no need to pass connection information, it's
        // picked up from configuration file based on the name of the
        // schema
        Northwind _db = new Northwind();

        // track previously selected category and product objects so
        // we can implement delete
        Category _previousCategory = null;
        Category _currentCategory = null;
        Product _previousProduct = null;
        Product _currentProduct = null;

        public WinAppSample() {
            InitializeComponent();
        }

        private void WinAppSample_Load(object sender, EventArgs e) {
            LoadCategories();
        }

        // the current category changed, the products for this category from
        // the category object. We'll load only if we didn't load before.
        private void categoryBindingSource_CurrentChanged(object sender, EventArgs e) {
            // track current and previous selection so we know which one
            // to delete when we get an event
            _previousCategory = _currentCategory;
            _currentCategory = (Category)categoryBindingSource.Current;

            // load the products if we didn't load them before
            if(!_currentCategory.Products.IsLoaded) {

                // we can only load the related items when an entity is no
                // in pending delete or pending add state (i.e. only Modified and Unchanged
                // states are valid for this)
                if(_currentCategory.EntityState == EntityState.Unchanged ||
                   _currentCategory.EntityState == EntityState.Modified) {
                    // NOTE: this does a database round-trip and is a common failure point
                    _currentCategory.Products.Load();
                }
            }
            
            // bind the products collection to the products binding source
            productBindingSource.DataSource = _currentCategory.Products.Source.Where("it is of (only NorthwindLib.Product)");
        }


        private void categoryBindingSource_AddingNew(object sender, AddingNewEventArgs e) {
            // the user just added a new category row to the grid, let the context know, so when
            // we call SaveChanges the category object gets pushed to the database.
            Category newCategory = new Category();
            e.NewObject = newCategory;
            _db.AddObject(newCategory);
        }

        private void productBindingSource_CurrentChanged(object sender, EventArgs e) {
            // track current and previous selection so we know which one
            // to delete when we get an event
            _previousProduct = _currentProduct;
            _currentProduct = (Product)productBindingSource.Current;
        }
        
        private void productBindingSource_AddingNew(object sender, AddingNewEventArgs e) {
            // the user just added a new product. We need to relate the product to the
            // current category. Note that since we're adding it to the related collection
            // in the category object, we don't need to also call _db.AddObject()
            Category category = (Category)categoryBindingSource.Current;
            Product newProduct = new Product();
            e.NewObject = newProduct;
            category.Products.Add(newProduct);
        }



        private void exitToolStripMenuItem_Click(object sender, EventArgs e) {
            Application.Exit();
        }

        private void saveChangesToolStripMenuItem_Click(object sender, EventArgs e) {
            // user requested that we save changes that have been made
            _db.SaveChanges();
        }

        private void btnRefreshCategories_Click(object sender, EventArgs e) {
            LoadCategories();
            categoryDataGridView.Refresh();
        }

        private void btnRefreshProducts_Click(object sender, EventArgs e) {
            Category category = (Category)categoryBindingSource.Current;
            if(null != category) {
                // call Load() again to refresh the list of products related to 
                // the current category
                category.Products.Load();
                productDataGridView.Refresh();
            }
        }

        void LoadCategories() {
                        
            categoryBindingSource.DataSource = _db.Categories;
        }

 
    }
}
