' Copyright (c) Microsoft Corporation.  All rights reserved.

'------------------------------------------------------------------------------
' PLEASE NOTE:  Microsoft Corporation (or based on where you live, one of its affiliates) 
' licenses this supplement to you.  You may use it with each validly licensed copy of Microsoft 
' Visual Studio Pre-Release Code-Name "ORCAS" software (the “software”).  You may not use the 
' supplement if you do not have a license for the software.  The license terms for the software 
' apply to your use of this supplement. 
'
' Please note: As this software is distributed in Quebec, Canada, this notice is provided below in French.
' Remarque: Ce logiciel étant distribué au Québec, Canada, les termes de cette licence sont fournis ci-dessous en français.
' REMARQUE:  Microsoft Corporation (ou en fonction du lieu où vous vivez, l’un de ses affiliés) vous 
' accorde une licence pour ce supplément.  Vous pouvez utiliser une copie de ce supplément avec chaque 
' copie concédée sous licence du logiciel Microsoft Visual Studio Pre-Release Code-Name "ORCAS" (le « logiciel »).  
' Vous n’êtes pas autorisé à utiliser ce supplément si vous n’êtes pas titulaire d’une licence pour le logiciel.  
' Les conditions de licence pour le logiciel s’appliquent à l’utilisation de ce supplément.  
'------------------------------------------------------------------------------
Module Module1

    ' Note the use of named connections in the connection string. This points
    ' to the actual connection string in the config file
    Const CONNECTION_STRING = "Name=Northwind"

    Sub Main()
		SimpleQuery()
        QueryWithParams()
        FetchWholeEntities()
        Inheritance_PolymorphicResultSets()
        Inheritance_FilterByType()
		RelationshipTraversal()
		CrossApply()
		CrossApplyWithAggregate()
        RegularInnerJoin()
        NestedResultSets()
        Aggregate()
    End Sub

    ' Execute a really simple query against the EDM schema to
    ' fetch all the Category entities from the Categories entity set
    Sub SimpleQuery()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			Dim cmd As New EntityCommand( _
			 "SELECT c.CategoryName, c.Description " & _
			 "FROM Northwind.Categories AS c " & _
			 "ORDER BY c.CategoryName", _
			  con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				Console.WriteLine("{0}" & vbTab & "{1}", r(0), r(1))
			Loop
		End Using

        Separator()
    End Sub

    ' Use parameters in a query to include external values such as user input into a query
    Sub QueryWithParams()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			Dim cmd As New EntityCommand( _
			 "SELECT c.CategoryName, c.Description " & _
			 "FROM Northwind.Categories AS c " & _
			 "WHERE c.CategoryID = @id", _
			 con)

			' parameters in the EntityCommand are like parameters in any other ADO.NET provider, just
			' create them through the parameters collection. The only difference is that parameter
			' names shouldn't include the "@" sign
			cmd.Parameters.AddWithValue("id", 1) ' 1 is a hardcoded ID that's always there in the sample DB

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				Console.WriteLine("{0}" & vbTab & "{1}", r(0), r(1))
			Loop
		End Using

        Separator()
    End Sub

    ' Use Entity SQL constructs to select whole entities instead of projecting
    ' individual properties (columns)
    Sub FetchWholeEntities()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' instead of projecting out individual members, we can request for the whole
			' entity to be fetched
			Dim cmd As New EntityCommand( _
			"SELECT c FROM Northwind.Categories AS c", _
			con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				' since we projected a single column "c" in the SELECT statement, we get a
				' single column back this column has a structured value, which is the 
				' Category entity. So the type of the row will be DbDataRecord and the
				' record will contain a column for each member of the Category entity type.
				Dim cat As DbDataRecord = CType(r("c"), DbDataRecord)

				Console.WriteLine("{0}" & vbTab & "{1}", cat("CategoryName"), cat("Description"))
			Loop
			r.Close()

			Console.WriteLine()
			Console.WriteLine("Now without the record wrapper around each entity:")

			' when projecting a single column you can choose to eliminate the record
			' wrapper by using the "VALUE" Entity SQL keyword, for example:
			cmd.CommandText = "SELECT VALUE c FROM Northwind.Categories AS c"

			r = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				' this time the record wrapper is not there, so each of the properties
				' of the Category entity is presented as a top-level column
				Console.WriteLine("{0}" & vbTab & "{1}", r("CategoryName"), r("Description"))
			Loop
		End Using

        Separator()
    End Sub

    ' The Products entity-set has instances of the Product and DiscontinuedProduct
    ' entity types (DiscontinuedProduct is a subtype of Product). Formulate a query
    ' that returns instances of both and discriminate them in the client
    Sub Inheritance_PolymorphicResultSets()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			Dim cmd As New EntityCommand( _
			"SELECT VALUE p " & _
			"FROM Northwind.Products AS p", _
			con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				' if we're only accessing properties that are defined in the most base
				' type (e.g. Product in this case), then no special casing is needed however,
				' here we want to access a property that's only available for a specific
				' subtype (the UnitsInStock property that's only in DiscontinuedProduct). So
				' we use the record metadata to find out the type of each entity

				' IExtendedDataRecord gets us access to the metadata information
				Dim productRecord As IExtendedDataRecord = CType(r, IExtendedDataRecord)

				' Print out properties that are present in all instances of Product or its subtypes
				Console.Write("{0}" & vbTab & "{1}", productRecord("ProductName"), productRecord("UnitPrice"))

				' if it's DiscontinuedProduct then print out extra information, otherwise
				' just output a new line
				If productRecord.DataRecordInfo.Metadata.EdmType.Name = "DiscontinuedProduct" Then
					Console.WriteLine(vbTab & "{0}", r("UnitsInStock"))
				Else
					Console.WriteLine()
				End If
			Loop
		End Using

        Separator()
    End Sub

    ' The Products entity-set has instances of the Product and DiscontinuedProduct
    ' entity types (DiscontinuedProduct is a subtype of Product). Formulate a query
    ' that returns only instances of DiscontinuedProduct or one of its subtypes
    Sub Inheritance_FilterByType()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' Use the "IS OF" Entity SQL operator to filter entities in
			' the Products entity-set that are of type DiscontinuedProducts or
			' anyo of its subtypes
			Dim cmd As New EntityCommand( _
			"SELECT VALUE p " & _
			"FROM Northwind.Products AS p " & _
			"WHERE p IS OF (NorthwindLib.DiscontinuedProduct)", _
			con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				' Since the query will return only DiscontinuedProduct instances, we
				' can access the UnitsInStock property without checking for the type
				' in the client
				Console.WriteLine("{0}" & vbTab & "{1}" & vbTab & "{2}", r("ProductName"), r("UnitPrice"), r("UnitsInStock"))
			Loop
		End Using

        Separator()
    End Sub

    ' Use the NAVIGATE operator to traverse the relationship between categories and
    ' products that's described in the model (the association is called "Category_Product"
    ' in the EDM model)
    Sub RelationshipTraversal()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' Find all products from the "Beverages" category 
			' NOTE: we're actually looking at supporting using "." for traversing
			' relationships instead of having to use the NAVIGATE operator
			Dim cmd As New EntityCommand( _
			"SELECT VALUE p " & _
			"FROM Northwind.Products AS p " & _
			"WHERE NAVIGATE(p, NorthwindLib.Category_Product).CategoryName = @name", _
			con)
			cmd.Parameters.AddWithValue("name", "Beverages")

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				Console.WriteLine("{0}", r("ProductName"))
			Loop
		End Using

        Separator()
    End Sub

	' Use the CROSS APPLY operator in the FROM clause as the equivalent of INNER JOIN when
	' an association from the entity on the left is defined to the entity on the right.
	Sub CrossApply()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' Find all products in each category category
			Dim cmd As New EntityCommand( _
				"SELECT c.CategoryName, p.ProductName " & _
				"FROM Northwind.Categories AS c " & _
				"CROSS APPLY c.Products AS p ", _
				con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				Console.WriteLine(r("CategoryName") & vbTab & r("ProductName"))
			Loop
		End Using

		Separator()
	End Sub

	' Use the CROSS APPLY operator in the FROM clause as the equivalent of INNER JOIN when
	' an association from the entity on the left is defined to the entity on the right, with
	' additional aggregation.
	Sub CrossApplyWithAggregate()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' Find the average unit price of products in each category category
			Dim cmd As New EntityCommand( _
				"SELECT c.CategoryName, SqlServer.AVG( p.UnitPrice ) as aup " & _
				"FROM Northwind.Categories AS c " & _
				"CROSS APPLY c.Products AS p " & _
				"GROUP BY c.CategoryName", _
				con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				Console.WriteLine(r("CategoryName") & vbTab & r("aup"))
			Loop
		End Using

		Separator()
	End Sub

	' If a relationship is not available for a given scenario, or an ad-hoc
	' join is needed for some other kind of query, regular INNER/OUTER joins
	' are available in Entity SQL. This example does the same as RelationshipTraversal()
	' but using an INNER JOIN
	Sub RegularInnerJoin()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' Find all products from the "Beverages" category
			' NOTE: the ExeuteReader call below is commented out, because this sample
			' won't run against the sample database this is because the sample EDM
			' schema is modeled using EDM relationships, not primary/foreign keys, so
			' there is no CategoryID property in the Product entity.
			Dim cmd As New EntityCommand( _
			"SELECT VALUE p " & _
			"FROM Northwind.Products AS p " & _
			"INNER JOIN Northwind.Categories AS c " & _
			  "ON p.CategoryID = c.CategoryID " & _
			"WHERE c.CategoryName = @name", _
			con)
			cmd.Parameters.AddWithValue("name", "Beverages")

			' Entity command requires SequentialAccess
			''dim r as DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			''do while r.Read()
			'    Console.WriteLine("{0}", r("ProductName"))
			''loop
		End Using

		Separator()
	End Sub

	' Obtain information about categories and products by formulating a 
	' query that returns a nested result-set containing both Category and
	' Product instances
	Sub NestedResultSets()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' In Entity SQL you can project out a collection (such as the results
			' of a nested subquery) and they show up as nested results in the resulting
			' datareader. Also, note that if we only used the result of NAVIGATE we would
			' only receive the references (key information) to the nested items the DEREF
			' operator de-references the references into the actual entities.
			Dim cmd As New EntityCommand( _
			"SELECT c, (SELECT VALUE DEREF(p) " & _
			   "FROM NAVIGATE(c, NorthwindLib.Category_Product) AS p) AS Products " & _
			"FROM Northwind.Categories AS c", _
			con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				' the top-level result-set has two columns, one contains the Category entity and
				' the other contains a datareader containing all the products that we selected for
				' that entity (which, based on the query will be the products associated with that
				' particular category)

				' Get the category record and print out the category name
				Dim categoryRec As DbDataRecord = CType(r.GetValue(0), DbDataRecord)
				Console.WriteLine(categoryRec("CategoryName"))

				' Now iterate over the products for this category row
				Dim products As DbDataReader = r.GetData(1)
				Do While products.Read()
					Console.WriteLine(vbTab & products("ProductName"))
				Loop
			Loop
		End Using

		Separator()
	End Sub

	Sub Aggregate()
		Using con As New EntityConnection(CONNECTION_STRING)
			con.Open()

			' In Entity SQL you can project out a collection (such as the results
			' of a nested subquery) and they show up as nested results in the resulting
			' datareader. Also, note that if we only used the result of NAVIGATE we would
			' only receive the references (key information) to the nested items the DEREF
			' operator de-references the references into the actual entities.
			Dim cmd As New EntityCommand( _
			"SELECT Name, SqlServer.COUNT(1) AS Count " & _
			"FROM Northwind.Products AS p " & _
			"GROUP BY NAVIGATE(p, NorthwindLib.Category_Product).CategoryName AS Name", _
			con)

			' Entity command requires SequentialAccess
			Dim r As DbDataReader = cmd.ExecuteReader(CommandBehavior.SequentialAccess)

			Do While r.Read()
				Console.WriteLine("{0}" & vbTab & "{1}", r(0), r(1))
			Loop
		End Using

		Separator()
	End Sub

	Sub Separator()
		Console.WriteLine("------------------------------------------------")
	End Sub

End Module
